package com.docus.soap.api;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.oxm.Marshaller;
import org.springframework.ws.WebServiceMessage;
import org.springframework.ws.client.core.WebServiceMessageCallback;
import org.springframework.ws.soap.saaj.SaajSoapMessage;
import org.springframework.ws.support.MarshallingUtils;

import javax.xml.transform.TransformerException;
import java.io.IOException;

public class SoapAPIMessageGenerator {
    private final Logger logger = LoggerFactory.getLogger(SoapAPIMessageGenerator.class);
    private Marshaller marshaller;

    public WebServiceMessageCallback generateSoapMessage(final SoapMessageEnvelope envelope) {
        return new WebServiceMessageCallback() {
            @Override
            public void doWithMessage(WebServiceMessage message) throws IOException, TransformerException {
                try {
                    SaajSoapMessage soapMessage = (SaajSoapMessage) message;
                    marshallerRequestSoapBody(soapMessage, envelope.getSoapBody());
                    soapMessage.setSoapAction(envelope.getSoapAction());
                } catch (Exception e) {
                    logger.error(e.getMessage(), e);
                }
            }
        };
    }

    private void marshallerRequestSoapBody(SaajSoapMessage soapMessage, Object soapBody) throws IOException {
        if (null == marshaller) {
            throw new IllegalStateException("Not marshaller registered ,please check your configuration.");
        }
        MarshallingUtils.marshal(marshaller, soapBody, soapMessage);
    }

    public void setMarshaller(Marshaller marshaller) {
        this.marshaller = marshaller;
    }
}
